import java.io.IOException;
import java.util.ArrayList;
import javafx.animation.Animation;
import javafx.animation.Interpolator;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.application.Application;
import javafx.geometry.Point3D;
import javafx.scene.AmbientLight;
import javafx.scene.Group;
import javafx.scene.PerspectiveCamera;
import javafx.scene.PointLight;
import javafx.scene.Scene;
import javafx.scene.input.KeyCode;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Box;
import javafx.scene.transform.Rotate;
import javafx.stage.Stage;
import javafx.util.Duration;

/**
 * Demo 3D animated sprite
 * 
 * @author mike slattery
 * @version may 2018
 */
public class SeagullWorld extends Application {

	private PerspectiveCamera camera;
	private Group cameraDolly;
	private final double cameraQuantity = 10.0;
	private final double sceneWidth = 600;
	private final double sceneHeight = 600;
	private final int FPS = 5; //Flaps per second!! (really "look" changes)
	
	int state = 0;
	ArrayList<Group> looks = new ArrayList<Group>();
	String[] lookFiles = {"bent", "up", "glide", "down"};

	private double mousePosX;
	private double mousePosY;
	private double mouseOldX;
	private double mouseOldY;
	private double mouseDeltaX;
	private double mouseDeltaY;

	private void constructWorld(Group root) {
		
		// AmbientLight light = new AmbientLight();
		AmbientLight light = new AmbientLight(Color.rgb(100, 100, 100));
		root.getChildren().add(light);

		//PointLight pl = new PointLight();
		PointLight pl = new PointLight(Color.RED);
		pl.setTranslateX(500);
		pl.setTranslateY(-180);
		pl.setTranslateZ(-200);
		root.getChildren().add(pl);

		final PhongMaterial greenMaterial = new PhongMaterial();
		greenMaterial.setDiffuseColor(Color.FORESTGREEN);
		greenMaterial.setSpecularColor(Color.LIMEGREEN);
		Box xAxis = new Box(500, 10, 10);
		xAxis.setMaterial(greenMaterial);
		Box yAxis = new Box(10, 200, 10);
		yAxis.setMaterial(greenMaterial);
		Box zAxis = new Box(10, 10, 200);
		zAxis.setMaterial(greenMaterial);
		root.getChildren().addAll(xAxis, yAxis, zAxis);
		
		
		// Load in the models for the various positions of the
		// gull.  They go into the list "looks" and also as
		// children of "orbit"
		Group orbit = new Group();
		for (int i = 0; i < lookFiles.length; i++)
		{
			ObjView drvr = new ObjView();
			try {
				drvr.load(ClassLoader.getSystemResource(lookFiles[i]+".obj").toString());
			} catch (IOException e) {
				System.out.println("Trouble loading model");
				e.printStackTrace();
			}
			Group gull = drvr.getRoot();
			gull.setScaleX(30);
			gull.setScaleY(-30);
			gull.setScaleZ(-30);
			gull.setRotationAxis(Rotate.Y_AXIS);
			gull.setRotate(90);
			gull.setTranslateX(400);
			gull.setTranslateY(-150);
			
			looks.add(gull);
			orbit.getChildren().add(gull);
		}	
		// Turn off extra images
		for (int i = 1; i < 4; i++)
			looks.get(i).setVisible(false);
		
		// We can't specify the center point (pivot) of the rotation
		// with a RotateTransition.  So, we introduce an explicit
		// Rotate transition and add it to the list of transforms
		// for the Node orbit.
		Rotate motor = new Rotate();
		motor.setAxis(new Point3D(0,-1,0));
		motor.setPivotX(0.0);
		motor.setPivotZ(0.0);
		orbit.getTransforms().add(motor);
		
		OrbitTransition rt = new OrbitTransition(Duration.millis(4000), motor);
		rt.setInterpolator(Interpolator.LINEAR);
		rt.setCycleCount(Animation.INDEFINITE);
		rt.play();
		
		root.getChildren().add(orbit);
	}
	
	void update()
	{
		looks.get(state).setVisible(false);
		state = (state + 1) % 4;
		looks.get(state).setVisible(true);	
	}

	@Override
	public void start(Stage primaryStage) {

		// Build your Scene and Camera
		Group sceneRoot = new Group();
		constructWorld(sceneRoot);

		Scene scene = new Scene(sceneRoot, sceneWidth, sceneHeight, true);
		scene.setFill(Color.BLACK);
		camera = new PerspectiveCamera(true);
		camera.setNearClip(0.1);
		camera.setFarClip(10000.0);
		scene.setCamera(camera);
		// translations through dolly
		cameraDolly = new Group();
		cameraDolly.setTranslateZ(-1000);
		cameraDolly.setTranslateX(200);
		cameraDolly.getChildren().add(camera);
		sceneRoot.getChildren().add(cameraDolly);
		// rotation transforms
		Rotate xRotate = new Rotate(0, 0, 0, 0, Rotate.X_AXIS);
		Rotate yRotate = new Rotate(0, 0, 0, 0, Rotate.Y_AXIS);
		camera.getTransforms().addAll(xRotate, yRotate);

		// Use keyboard to control camera position
		scene.setOnKeyPressed(event -> {
			double change = cameraQuantity;
			// What key did the user press?
			KeyCode keycode = event.getCode();

			Point3D delta = null;
			if (keycode == KeyCode.COMMA) {
				delta = new Point3D(0, 0, change);
			}
			if (keycode == KeyCode.PERIOD) {
				delta = new Point3D(0, 0, -change);
			}
			if (keycode == KeyCode.A) {
				delta = new Point3D(-change, 0, 0);
			}
			if (keycode == KeyCode.D) {
				delta = new Point3D(change, 0, 0);
			}
			if (keycode == KeyCode.W) {
				delta = new Point3D(0, -change, 0);
			}
			if (keycode == KeyCode.S) {
				delta = new Point3D(0, change, 0);
			}
			if (delta != null) {
				Point3D delta2 = camera.localToParent(delta);
				cameraDolly.setTranslateX(cameraDolly.getTranslateX() + delta2.getX());
				cameraDolly.setTranslateY(cameraDolly.getTranslateY() + delta2.getY());
				cameraDolly.setTranslateZ(cameraDolly.getTranslateZ() + delta2.getZ());

			}
		});

		// Use mouse to control camera rotation
		scene.setOnMousePressed(me -> {
			mousePosX = me.getSceneX();
			mousePosY = me.getSceneY();
		});

		scene.setOnMouseDragged(me -> {
			mouseOldX = mousePosX;
			mouseOldY = mousePosY;
			mousePosX = me.getSceneX();
			mousePosY = me.getSceneY();
			mouseDeltaX = (mousePosX - mouseOldX);
			mouseDeltaY = (mousePosY - mouseOldY);

			yRotate.setAngle(((yRotate.getAngle() - mouseDeltaX * 0.2) % 360 + 540) % 360 - 180); // +
			xRotate.setAngle(((xRotate.getAngle() + mouseDeltaY * 0.2) % 360 + 540) % 360 - 180); // -
		});

		// Setup and start animation loop (Timeline)
				KeyFrame kf = new KeyFrame(Duration.millis(1000 / FPS),
						e -> {
							// update position
							update();
						}
					);
				Timeline mainLoop = new Timeline(kf);
				mainLoop.setCycleCount(Animation.INDEFINITE);
				mainLoop.play();

		primaryStage.setTitle("Flying");
		primaryStage.setScene(scene);
		primaryStage.show();
	}

	public static void main(String[] args) {
		launch(args);
	}
}